#-*- coding: UTF-8 -*-

# ---------------------------------------------------------
# Un analyseur lexical pour le lambda-calcul
# Marc Lorenzi - 27 mai 2018
# ---------------------------------------------------------


def is_var(x): return 'a' <= x and x <= 'z'
def is_maj(x): return 'A' <= x and x <= 'Z'
def is_digit(x): return '0' <= x and x <= '9'
def is_space(x): return x == ' '
def is_lpar(x): return x == '('
def is_rpar(x): return x == ')'

def is_lambda(x): return x == '#'
def is_dot(x): return x == '.'

class Lexer:

    def __init__(self, s):
        self.s = s
        self.r = []
        self.k = 0
        self.length = len(s)

    def get_lexeme(self):
        self.skip_spaces()
        if self.k == len(self.s): return
        x = self.s[self.k]
        if is_digit(x): self.get_number()
        elif is_var(x): self.get_var()
        elif is_maj(x): self.get_alias()
        elif is_lpar(x): self.get_lpar()
        elif is_rpar(x): self.get_rpar()
        elif is_lambda(x): self.get_lambda()
        elif is_dot(x): self.get_dot()
        else: raise Exception('Unknown character')

    def lex(self):
        while self.k < len(self.s):
            self.get_lexeme()

    def skip_spaces(self):
        while self.k < self.length and is_space(self.s[self.k]):
            self.k = self.k + 1

    def get_number(self):
        n = 0
        ind = self.k
        while self.k < self.length and is_digit(self.s[self.k]):
            n = 10 * n + ord(self.s[self.k]) - ord('0')
            self.k = self.k + 1
        self.r.append(['number', n, ind])

    def get_var(self):
        self.r.append(['var', self.s[self.k], self.k])
        self.k = self.k + 1

    def get_alias(self):
        s1 = ''
        ind = self.k
        while self.k < self.length and is_maj(self.s[self.k]):
            s1 = s1 + self.s[self.k]
            self.k = self.k + 1
        self.r.append(['alias', s1, ind])

    def get_lpar(self):
        self.r.append(['lpar', '(', self.k])
        self.k = self.k + 1

    def get_rpar(self):
        self.r.append(['rpar', ')', self.k])
        self.k = self.k + 1

    def get_lambda(self):
        self.r.append(['lambda', '#', self.k])
        self.k = self.k + 1

    def get_dot(self):
        self.r.append(['dot', '.', self.k])
        self.k = self.k + 1

# ---------------------------------------------------------
# La fonction d'analyse lexicale

def lex(s):
    L = Lexer(s)
    L.lex()
    return L.r
        

